<?php

namespace Islhd\Order\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Islhd\Order\Http\Requests\DetailRequest;
use Islhd\Order\Interfaces\DetailRepositoryInterface;
use Islhd\Order\Models\Detail;

/**
 * Resource controller class for detail.
 */
class DetailResourceController extends BaseController
{

    /**
     * Initialize detail resource controller.
     *
     * @param type DetailRepositoryInterface $detail
     *
     * @return null
     */
    public function __construct(DetailRepositoryInterface $detail)
    {
        parent::__construct();
        $this->repository = $detail;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Islhd\Order\Repositories\Criteria\DetailResourceCriteria::class);
    }

    /**
     * Display a list of detail.
     *
     * @return Response
     */
    public function index(DetailRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Islhd\Order\Repositories\Presenter\DetailPresenter::class)
                ->$function();
        }

        $details = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('order::detail.names'))
            ->view('order::detail.index', true)
            ->data(compact('details', 'view'))
            ->output();
    }

    /**
     * Display detail.
     *
     * @param Request $request
     * @param Model   $detail
     *
     * @return Response
     */
    public function show(DetailRequest $request, Detail $detail)
    {

        if ($detail->exists) {
            $view = 'order::detail.show';
        } else {
            $view = 'order::detail.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('order::detail.name'))
            ->data(compact('detail'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new detail.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DetailRequest $request)
    {

        $detail = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('order::detail.name')) 
            ->view('order::detail.create', true) 
            ->data(compact('detail'))
            ->output();
    }

    /**
     * Create new detail.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DetailRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $detail                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('order::detail.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('order/detail/' . $detail->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/order/detail'))
                ->redirect();
        }

    }

    /**
     * Show detail for editing.
     *
     * @param Request $request
     * @param Model   $detail
     *
     * @return Response
     */
    public function edit(DetailRequest $request, Detail $detail)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('order::detail.name'))
            ->view('order::detail.edit', true)
            ->data(compact('detail'))
            ->output();
    }

    /**
     * Update the detail.
     *
     * @param Request $request
     * @param Model   $detail
     *
     * @return Response
     */
    public function update(DetailRequest $request, Detail $detail)
    {
        try {
            $attributes = $request->all();

            $detail->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('order::detail.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('order/detail/' . $detail->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('order/detail/' . $detail->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the detail.
     *
     * @param Model   $detail
     *
     * @return Response
     */
    public function destroy(DetailRequest $request, Detail $detail)
    {
        try {

            $detail->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('order::detail.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('order/detail/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('order/detail/' . $detail->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple detail.
     *
     * @param Model   $detail
     *
     * @return Response
     */
    public function delete(DetailRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('order::detail.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('order/detail'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/order/detail'))
                ->redirect();
        }

    }

    /**
     * Restore deleted details.
     *
     * @param Model   $detail
     *
     * @return Response
     */
    public function restore(DetailRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('order::detail.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/order/detail'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/order/detail/'))
                ->redirect();
        }

    }

}
